#!python3
# -*- coding: utf-8 -*-

""" Author: nicolai92 """

import plotly.graph_objects as go
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
import math
import re
from sklearn.tree import DecisionTreeClassifier
from sklearn.model_selection import train_test_split
from sklearn import metrics
from sklearn import tree
import pydotplus


sns.set(style="darkgrid")


def create_analysis(cols, query):
    # Filter dataframe
    analysis = df[cols][query]
    # Calculate mean value (group by gender)
    mean_df = analysis.groupby('gender', as_index=True)[cols].mean()
    return mean_df


def create_spider_plot(df, title: str = None) -> go.Figure:
    # Use a list of columns for labeling
    cols = list(df.columns)
    # For better readability, rename columns
    res = [re.compile(r"\d{1}[_]{1}\d{1}").sub("", m) for m in cols]
    # print(res)
    dic = {'attr': 'Attractiveness', 'sinc': 'Sincerity', 'intel': 'Intelligence',
           'fun': 'Fun', 'amb': 'Ambition', 'shar': 'Shared Interests / Hobbies'}
    res = [dic.get(n, n) for n in res]

    # Create a Spider Chart
    fig = go.Figure()

    # Female Spider Chart
    fig.add_trace(go.Scatterpolar(
        r=df.loc['female'].values.tolist(),  # List of points for attributes
        theta=res,  # Labels for attributes
        fill='toself',
        line_color='#9CBDE6',  # Use pastel colors of Seaborn lib.
        name='Female'
    ))
    # Male Spider Chart
    fig.add_trace(go.Scatterpolar(
        r=df.loc['male'].values.tolist(),
        theta=res,
        fill='toself',
        line_color='#E7A77D',
        name='Male'
    ))
    # Set Title and Value Range of Spider Chart
    fig.update_layout(
        title=title,
        polar=dict(
            radialaxis=dict(
                visible=False
            )),
        showlegend=True
    )
    return fig


def create_decision_tree():

    df_men = df[df['gender'] == 'male']

    # Interpolate missing values
    df_men.interpolate(method='pad', inplace=True)

    # Features
    feature_cols = ['attr_o', 'sinc_o', 'intel_o', 'fun_o', 'amb_o', 'shar_o']
    X = df_men[feature_cols]

    # Target
    Y = df_men.dec_o

    # Split into training (70%) and test (30%)
    X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size=0.3, random_state=1)  # noqa

    # Create Decision Tree
    clf = DecisionTreeClassifier(max_depth=2)
    clf = clf.fit(X_train, Y_train)
    Y_pred = clf.predict(X_test)

    # Accuracy
    print("Accuracy: ", metrics.accuracy_score(Y_test, Y_pred))

    # Create dot data
    dot_data = tree.export_graphviz(clf, out_file=None,
                                    feature_names=feature_cols,
                                    class_names=['0', '1'],
                                    filled=True, rounded=True)

    # Draw graph
    graph = pydotplus.graph_from_dot_data(dot_data)
    graph.write_png("07_decision_tree_men.png")


# Read local file
fd = 'Speed Dating Data.csv'
df = pd.read_csv(fd, delimiter=',', encoding='ISO-8859-1')

# Changes to the Dataset
df.gender.replace([1, 0], ['male', 'female'], inplace=True)

# Male vs. Female
cols = ['iid']
male_vs_female = df.groupby('gender', as_index=True)[cols].nunique()

# ax = sns.barplot(data=male_vs_female.reset_index(),
#                  x='gender', y='iid', palette='pastel')

# ax.set_title('How many people participated in the Speed Dating Events?')
# ax.set_ylabel('participants')
# ax.set_xlabel('gender')

# plt.savefig('01_male_vs_female.png')

# Male vs. Female by age
age_distribution = df.groupby(['gender', 'age'], as_index=True)[
    cols].nunique()

# bx = sns.boxplot(data=age_distribution.reset_index(),
#                  x='gender', y='age', palette='pastel')

# bx.set_title('How old were participants by gender?')
# bx.set_ylabel('age')
# bx.set_xlabel('gender')

# plt.savefig('02_male_vs_female_age.png')

# Spider Chart - What are participants looking for?
cols = ['gender', 'attr1_1', 'sinc1_1', 'intel1_1', 'fun1_1', 'amb1_1', 'shar1_1']  # noqa
query = df['wave'].between(1, 5, inclusive=True) | df['wave'].between(10, 21, inclusive=True)  # noqa

df_analysis = create_analysis(cols, query)
fig = create_spider_plot(df=df_analysis)

# fig.show()
# fig.write_image("fig1.png")

# Spider Chart - What participants think their same-sex peer is looking for?
cols = ['gender', 'attr4_1', 'sinc4_1', 'intel4_1', 'fun4_1', 'amb4_1', 'shar4_1']  # noqa
query = df['wave'].between(10, 21, inclusive=True)  # noqa

df_analysis = create_analysis(cols, query)
fig = create_spider_plot(df=df_analysis)

# fig.show()
# fig.write_image("fig2.png")

# Spider Chart - What participants think their same-sex peer is looking for?
cols = ['gender', 'attr2_1', 'sinc2_1', 'intel2_1', 'fun2_1', 'amb2_1', 'shar2_1']  # noqa
query = df['wave'].between(1, 5, inclusive=True) | df['wave'].between(10, 21, inclusive=True)  # noqa

df_analysis = create_analysis(cols, query)
fig = create_spider_plot(df=df_analysis)

# fig.show()
# fig.write_image("fig3.png")

# Replace numbers with strings for better readability
df.goal.replace([1, 2, 3, 4, 5, 6], ['Have Fun', 'Meet People',
                                     'Dates', 'Relationship', 'Say, I did it', 'Other'], inplace=True)

# Bar chart - Primary Goal in Speed Dating Event
cols = ['iid']
goals = df.groupby('goal', as_index=True)[cols].nunique()

# Create bar chart
cx = sns.barplot(data=goals.sort_values(['iid']).reset_index(),
                 x='goal', y='iid', palette='pastel')

cx.set_title('What is the primary goal in the Speed Dating Events?')
cx.set_ylabel('participants')
cx.set_xlabel('goal')

plt.savefig('06_primary_goal_in_speed_dating_events.png')

# Decision Tree - How to get a second date?
# create_decision_tree()
